@@ -51,9 +51,10 @@ import re
 import logging
 
 
-from ganeti import errors
-from ganeti import utils
 from ganeti import constants
+from ganeti import errors
+from ganeti import objects
+from ganeti import utils
 
 
 def _IsCpuMaskWellFormed(cpu_mask):
@@ -88,10 +89,16 @@ def _IsMultiCpuMaskWellFormed(cpu_mask):
 # Read the BaseHypervisor.PARAMETERS docstring for the syntax of the
 # _CHECK values
 
-# must be afile
+# must be a file
 _FILE_CHECK = (utils.IsNormAbsPath, "must be an absolute normalized path",
                os.path.isfile, "not found or not a file")
 
+# must be a file or a URL
+_FILE_OR_URL_CHECK = (lambda x: utils.IsNormAbsPath(x) or utils.IsUrl(x),
+                      "must be an absolute normalized path or a URL",
+                      lambda x: os.path.isfile(x) or utils.IsUrl(x),
+                      "not found or not a file or URL")
+
 # must be a directory
 _DIR_CHECK = (utils.IsNormAbsPath, "must be an absolute normalized path",
               os.path.isdir, "not found or not a directory")
@@ -111,16 +118,24 @@ _MULTI_CPU_MASK_CHECK = (_IsMultiCpuMaskWellFormed,
 _NET_PORT_CHECK = (lambda x: 0 < x < 65535, "invalid port number",
                    None, None)
 
+# Check if number of queues is in safe range
+_VIRTIO_NET_QUEUES_CHECK = (lambda x: 0 < x < 9, "invalid number of queues",
+                            None, None)
+
 # Check that an integer is non negative
 _NONNEGATIVE_INT_CHECK = (lambda x: x >= 0, "cannot be negative", None, None)
 
 # nice wrappers for users
 REQ_FILE_CHECK = (True, ) + _FILE_CHECK
 OPT_FILE_CHECK = (False, ) + _FILE_CHECK
+REQ_FILE_OR_URL_CHECK = (True, ) + _FILE_OR_URL_CHECK
+OPT_FILE_OR_URL_CHECK = (False, ) + _FILE_OR_URL_CHECK
 REQ_DIR_CHECK = (True, ) + _DIR_CHECK
 OPT_DIR_CHECK = (False, ) + _DIR_CHECK
 REQ_NET_PORT_CHECK = (True, ) + _NET_PORT_CHECK
 OPT_NET_PORT_CHECK = (False, ) + _NET_PORT_CHECK
+REQ_VIRTIO_NET_QUEUES_CHECK = (True, ) + _VIRTIO_NET_QUEUES_CHECK
+OPT_VIRTIO_NET_QUEUES_CHECK = (False, ) + _VIRTIO_NET_QUEUES_CHECK
 REQ_CPU_MASK_CHECK = (True, ) + _CPU_MASK_CHECK
 OPT_CPU_MASK_CHECK = (False, ) + _CPU_MASK_CHECK
 REQ_MULTI_CPU_MASK_CHECK = (True, ) + _MULTI_CPU_MASK_CHECK
@@ -153,6 +168,99 @@ def ParamInSet(required, my_set):
   return (required, fn, err, None, None)
 
 
+def GenerateTapName():
+  """Generate a TAP network interface name for a NIC.
+
+  This helper function generates a special TAP network interface
+  name for NICs that are meant to be used in instance communication.
+  This function checks the existing TAP interfaces in order to find
+  a unique name for the new TAP network interface.  The TAP network
+  interface names are of the form 'gnt.com.%d', where '%d' is a
+  unique number within the node.
+
+  @rtype: string
+  @return: TAP network interface name, or the empty string if the
+           NIC is not used in instance communication
+
+  """
+  result = utils.RunCmd(["ip", "tuntap", "list"])
+
+  if result.failed:
+    raise errors.HypervisorError("Failed to list TUN/TAP interfaces")
+
+  idxs = set()
+
+  for line in result.output.splitlines():
+    parts = line.split(": ", 1)
+
+    if len(parts) < 2:
+      raise errors.HypervisorError("Failed to parse TUN/TAP interfaces")
+
+    r = re.match(r"gnt\.com\.([0-9]+)", parts[0])
+
+    if r is not None:
+      idxs.add(int(r.group(1)))
+
+  if idxs:
+    idx = max(idxs) + 1
+  else:
+    idx = 0
+
+  return "gnt.com.%d" % idx
+
+
+def ConfigureNIC(cmd, instance, seq, nic, tap):
+  """Run the network configuration script for a specified NIC
+
+  @type cmd: string
+  @param cmd: command to run
+  @type instance: instance object
+  @param instance: instance we're acting on
+  @type seq: int
+  @param seq: nic sequence number
+  @type nic: nic object
+  @param nic: nic we're acting on
+  @type tap: str
+  @param tap: the host's tap interface this NIC corresponds to
+
+  """
+  env = {
+    "PATH": "%s:/sbin:/usr/sbin" % os.environ["PATH"],
+    "INSTANCE": instance.name,
+    "MAC": nic.mac,
+    "MODE": nic.nicparams[constants.NIC_MODE],
+    "INTERFACE": tap,
+    "INTERFACE_INDEX": str(seq),
+    "INTERFACE_UUID": nic.uuid,
+    "TAGS": " ".join(instance.GetTags()),
+  }
+
+  if nic.ip:
+    env["IP"] = nic.ip
+
+  if nic.name:
+    env["INTERFACE_NAME"] = nic.name
+
+  if nic.nicparams[constants.NIC_LINK]:
+    env["LINK"] = nic.nicparams[constants.NIC_LINK]
+
+  if constants.NIC_VLAN in nic.nicparams:
+    env["VLAN"] = nic.nicparams[constants.NIC_VLAN]
+
+  if nic.network:
+    n = objects.Network.FromDict(nic.netinfo)
+    env.update(n.HooksDict())
+
+  if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
+    env["BRIDGE"] = nic.nicparams[constants.NIC_LINK]
+
+  result = utils.RunCmd(cmd, env=env)
+  if result.failed:
+    raise errors.HypervisorError("Failed to configure interface %s: %s;"
+                                 " network configuration script output: %s" %
+                                 (tap, result.fail_reason, result.output))
+
+
 class HvInstanceState(object):
   RUNNING = 0
   SHUTDOWN = 1
