@@ -193,6 +193,8 @@ class SimpleStore(object):
     ssconf_lock.Exclusive(blocking=True, timeout=SSCONF_LOCK_TIMEOUT)
     try:
       for name, value in values.iteritems():
+        if isinstance(value, (list, tuple)):
+          value = "\n".join(value)
         if value and not value.endswith("\n"):
           value += "\n"
 
