@@ -38,15 +38,16 @@ import time
 
 from ganeti import utils
 from ganeti import constants
-from ganeti import query
 from ganeti import pathutils
+from ganeti import query
+from ganeti.netutils import IP4Address
 
 import qa_config
 import qa_daemon
 import qa_utils
 import qa_error
 
-from qa_utils import AssertCommand, AssertEqual
+from qa_utils import AssertCommand, AssertEqual, AssertIn
 from qa_utils import InstanceCheck, INST_DOWN, INST_UP, FIRST_ARG, RETURN_VALUE
 from qa_instance_utils import CheckSsconfInstanceList, \
                               CreateInstanceDrbd8, \
@@ -367,6 +368,40 @@ def TestInstanceReinstall(instance):
     print qa_utils.FormatInfo("Test not supported for diskless instances")
     return
 
+  qa_storage = qa_config.get("qa-storage")
+
+  if qa_storage is None:
+    print qa_utils.FormatInfo("Test not supported because the additional QA"
+                              " storage is not available")
+  else:
+    # Reinstall with OS image from QA storage
+    url = "%s/busybox.img" % qa_storage
+    AssertCommand(["gnt-instance", "reinstall",
+                   "--os-parameters", "os-image=" + url,
+                   "-f", instance.name])
+
+    # Reinstall with OS image as local file on the node
+    pnode = _GetInstanceField(instance.name, "pnode")
+
+    cmd = ("wget -O busybox.img %s &> /dev/null &&"
+           " echo $(pwd)/busybox.img") % url
+    image = qa_utils.GetCommandOutput(pnode, cmd).strip()
+
+    AssertCommand(["gnt-instance", "reinstall",
+                   "--os-parameters", "os-image=" + image,
+                   "-f", instance.name])
+
+  # Reinstall non existing local file
+  AssertCommand(["gnt-instance", "reinstall",
+                 "--os-parameters", "os-image=NonExistantOsForQa",
+                 "-f", instance.name], fail=True)
+
+  # Reinstall non existing URL
+  AssertCommand(["gnt-instance", "reinstall",
+                 "--os-parameters", "os-image=http://NonExistantOsForQa",
+                 "-f", instance.name], fail=True)
+
+  # Reinstall using OS scripts
   AssertCommand(["gnt-instance", "reinstall", "-f", instance.name])
 
   # Test with non-existant OS definition
@@ -375,6 +410,16 @@ def TestInstanceReinstall(instance):
                  instance.name],
                 fail=True)
 
+  # Test with existing OS but invalid variant
+  AssertCommand(["gnt-instance", "reinstall", "-f", "-o", "debootstrap+ola",
+                 instance.name],
+                fail=True)
+
+  # Test with existing OS but invalid variant
+  AssertCommand(["gnt-instance", "reinstall", "-f", "-o", "debian-image+ola",
+                 instance.name],
+                fail=True)
+
 
 @InstanceCheck(INST_DOWN, INST_DOWN, FIRST_ARG)
 def TestInstanceRenameAndBack(rename_source, rename_target):
@@ -566,14 +611,23 @@ def TestInstanceModify(instance):
       ])
   elif default_hv == constants.HT_KVM and \
     qa_config.TestEnabled("instance-device-hotplug"):
+    # FIXME: Fix issue 885 and then re-enable the tests below
+    #args.extend([
+    #  ["--net", "-1:add", "--hotplug"],
+    #  ["--net", "-1:modify,mac=aa:bb:cc:dd:ee:ff", "--hotplug", "--force"],
+    #  ["--net", "-1:remove", "--hotplug"],
+    #  ])
     args.extend([
-      ["--net", "-1:add", "--hotplug"],
-      ["--net", "-1:modify,mac=aa:bb:cc:dd:ee:ff", "--hotplug", "--force"],
-      ["--net", "-1:remove", "--hotplug"],
       ["--disk", "-1:add,size=1G", "--hotplug"],
       ["--disk", "-1:remove", "--hotplug"],
       ])
 
+  url = "http://example.com/busybox.img"
+  args.extend([
+      ["--os-parameters", "os-image=" + url],
+      ["--os-parameters", "os-image=default"]
+      ])
+
   for alist in args:
     AssertCommand(["gnt-instance", "modify"] + alist + [instance.name])
 
@@ -1249,7 +1303,7 @@ def _TestInstanceUserDownXen(instance):
 def _TestInstanceUserDownKvm(instance):
   def _StopKVMInstance():
     AssertCommand("pkill -f \"\\-name %s\"" % instance.name, node=primary)
-    time.sleep(5)
+    time.sleep(10)
 
   AssertCommand(["gnt-cluster", "modify", "--user-shutdown=true"])
   AssertCommand(["gnt-instance", "modify", "-H", "user_shutdown=true",
@@ -1285,6 +1339,113 @@ def TestInstanceUserDown(instance):
           % hv
 
 
+@InstanceCheck(INST_UP, INST_UP, FIRST_ARG)
+def TestInstanceCommunication(instance, master):
+  """Tests instance communication via 'gnt-instance modify'"""
+
+  # Enable instance communication network at the cluster level
+  network_name = "mynetwork"
+
+  cmd = ["gnt-cluster", "modify",
+         "--instance-communication-network=%s" % network_name]
+  result_output = qa_utils.GetCommandOutput(master.primary,
+                                            utils.ShellQuoteArgs(cmd))
+  print result_output
+
+  # Enable instance communication mechanism for this instance
+  AssertCommand(["gnt-instance", "modify", "-c", "yes", instance.name])
+
+  # Reboot instance for changes to NIC to take effect
+  AssertCommand(["gnt-instance", "reboot", instance.name])
+
+  # Check if the instance is properly configured for instance
+  # communication.
+  nic_name = "%s%s" % (constants.INSTANCE_COMMUNICATION_NIC_PREFIX,
+                       instance.name)
+
+  ## Check the output of 'gnt-instance list'
+  nic_names = _GetInstanceField(instance.name, "nic.names")
+  nic_names = map(lambda x: x.strip(" '"), nic_names.strip("[]").split(","))
+
+  AssertIn(nic_name, nic_names,
+           msg="Looking for instance communication TAP interface")
+
+  nic_n = nic_names.index(nic_name)
+
+  nic_ip = _GetInstanceField(instance.name, "nic.ip/%d" % nic_n)
+  nic_network = _GetInstanceField(instance.name, "nic.network.name/%d" % nic_n)
+  nic_mode = _GetInstanceField(instance.name, "nic.mode/%d" % nic_n)
+
+  AssertEqual(IP4Address.InNetwork(constants.INSTANCE_COMMUNICATION_NETWORK4,
+                                   nic_ip),
+              True,
+              msg="Checking if NIC's IP if part of the expected network")
+
+  AssertEqual(network_name, nic_network,
+              msg="Checking if NIC's network name matches the expected value")
+
+  AssertEqual(constants.INSTANCE_COMMUNICATION_NETWORK_MODE, nic_mode,
+              msg="Checking if NIC's mode name matches the expected value")
+
+  ## Check the output of 'ip route'
+  cmd = ["ip", "route", "show", nic_ip]
+  result_output = qa_utils.GetCommandOutput(master.primary,
+                                            utils.ShellQuoteArgs(cmd))
+  result = result_output.split()
+
+  AssertEqual(len(result), 5, msg="Checking if the IP route is established")
+
+  route_ip = result[0]
+  route_dev = result[1]
+  route_tap = result[2]
+  route_scope = result[3]
+  route_link = result[4]
+
+  AssertEqual(route_ip, nic_ip,
+              msg="Checking if IP route shows the expected IP")
+  AssertEqual(route_dev, "dev",
+              msg="Checking if IP route shows the expected device")
+  AssertEqual(route_scope, "scope",
+              msg="Checking if IP route shows the expected scope")
+  AssertEqual(route_link, "link",
+              msg="Checking if IP route shows the expected link-level scope")
+
+  ## Check the output of 'ip address'
+  cmd = ["ip", "address", "show", "dev", route_tap]
+  result_output = qa_utils.GetCommandOutput(master.primary,
+                                            utils.ShellQuoteArgs(cmd))
+  result = result_output.splitlines()
+
+  AssertEqual(len(result), 3,
+              msg="Checking if the IP address is established")
+
+  result = result.pop().split()
+
+  AssertEqual(len(result), 7,
+              msg="Checking if the IP address has the expected value")
+
+  address_ip = result[1]
+  address_netmask = result[3]
+
+  AssertEqual(address_ip, "169.254.169.254/32",
+              msg="Checking if the TAP interface has the expected IP")
+  AssertEqual(address_netmask, "169.254.255.255",
+              msg="Checking if the TAP interface has the expected netmask")
+
+  # Disable instance communication mechanism for this instance
+  AssertCommand(["gnt-instance", "modify", "-c", "no", instance.name])
+
+  # Reboot instance for changes to NIC to take effect
+  AssertCommand(["gnt-instance", "reboot", instance.name])
+
+  # Disable instance communication network at cluster level
+  cmd = ["gnt-cluster", "modify",
+         "--instance-communication-network=%s" % network_name]
+  result_output = qa_utils.GetCommandOutput(master.primary,
+                                            utils.ShellQuoteArgs(cmd))
+  print result_output
+
+
 available_instance_tests = [
   ("instance-add-plain-disk", constants.DT_PLAIN,
    TestInstanceAddWithPlainDisk, 1),
