@@ -59,7 +59,9 @@ import qualified Data.List as List
 import qualified Data.Map as Map
 import Data.Maybe (fromMaybe)
 import qualified Data.Set as Set
+import Data.Word (Word32)
 import GHC.Exts (IsString(..))
+import System.Time (ClockTime(..))
 import qualified Text.JSON as J
 
 import Test.Ganeti.TestHelper
@@ -69,6 +71,7 @@ import Test.Ganeti.Types ()
 import qualified Ganeti.Constants as C
 import Ganeti.Network
 import Ganeti.Objects as Objects
+import qualified Ganeti.Objects.BitArray as BA
 import Ganeti.JSON
 import Ganeti.Types
 
@@ -87,8 +90,14 @@ $(genArbitrary ''BlockDriver)
 
 $(genArbitrary ''DiskMode)
 
+instance Arbitrary LogicalVolume where
+  arbitrary = LogicalVolume <$> validName <*> validName
+    where
+      validName = -- we intentionally omit '.' and '-' to avoid forbidden names
+        listOf1 $ elements (['a'..'z'] ++ ['A'..'Z'] ++ ['0'..'9'] ++ "+_")
+
 instance Arbitrary DiskLogicalId where
-  arbitrary = oneof [ LIDPlain <$> arbitrary <*> arbitrary
+  arbitrary = oneof [ LIDPlain <$> arbitrary
                     , LIDDrbd8 <$> genFQDN <*> genFQDN <*> arbitrary
                                <*> arbitrary <*> arbitrary <*> arbitrary
                     , LIDFile  <$> arbitrary <*> arbitrary
@@ -102,7 +111,8 @@ instance Arbitrary DiskLogicalId where
 instance Arbitrary Disk where
   arbitrary = Disk <$> arbitrary <*> pure [] <*> arbitrary
                    <*> arbitrary <*> arbitrary <*> arbitrary
-                   <*> arbitrary <*> arbitrary
+                   <*> arbitrary <*> arbitrary <*> arbitrary
+                   <*> arbitrary <*> arbitrary <*> arbitrary
 
 -- FIXME: we should generate proper values, >=0, etc., but this is
 -- hard for partial ones, where all must be wrapped in a 'Maybe'
@@ -134,6 +144,8 @@ instance Arbitrary Instance where
       <*> arbitrary
       -- osparams
       <*> pure (GenericContainer Map.empty)
+      -- osparams_private
+      <*> pure (GenericContainer Map.empty)
       -- admin_state
       <*> arbitrary
       -- admin_state_source
@@ -141,7 +153,7 @@ instance Arbitrary Instance where
       -- nics
       <*> arbitrary
       -- disks
-      <*> vectorOf 5 genDisk
+      <*> vectorOf 5 arbitrary
       -- disk template
       <*> arbitrary
       -- disks active
@@ -195,9 +207,13 @@ genDiskWithChildren num_children = do
   mode <- arbitrary
   name <- genMaybe genName
   spindles <- arbitrary
+  params <- arbitrary
   uuid <- genName
-  let disk = Disk logicalid children ivname size mode name spindles uuid
-  return disk
+  serial <- arbitrary
+  time <- arbitrary
+  return $
+    Disk logicalid children ivname size mode name
+      spindles params uuid serial time time
 
 genDisk :: Gen Disk
 genDisk = genDiskWithChildren 3
@@ -232,6 +248,12 @@ instance Arbitrary ClusterNicParams where
 instance Arbitrary OsParams where
   arbitrary = (GenericContainer . Map.fromList) <$> arbitrary
 
+instance Arbitrary Objects.ClusterOsParamsPrivate where
+  arbitrary = (GenericContainer . Map.fromList) <$> arbitrary
+
+instance Arbitrary a => Arbitrary (Private a) where
+  arbitrary = Private <$> arbitrary
+
 instance Arbitrary ClusterOsParams where
   arbitrary = (GenericContainer . Map.fromList) <$> arbitrary
 
@@ -246,6 +268,9 @@ instance Arbitrary IAllocatorParams where
 
 $(genArbitrary ''Cluster)
 
+instance Arbitrary AddressPool where
+  arbitrary = AddressPool . BA.fromList <$> arbitrary
+
 instance Arbitrary Network where
   arbitrary = genValidNetwork
 
@@ -267,17 +292,18 @@ genValidNetwork = do
   uuid <- arbitrary
   ctime <- arbitrary
   mtime <- arbitrary
-  let n = Network name mac_prefix (Ip4Network net netmask) net6 gateway
+  let n = Network name mac_prefix (mkIp4Network net netmask) net6 gateway
           gateway6 res ext_res uuid ctime mtime 0 Set.empty
   return n
 
 -- | Generate an arbitrary string consisting of '0' and '1' of the given length.
-genBitString :: Int -> Gen String
-genBitString len = vectorOf len (elements "01")
+genBitString :: Int -> Gen AddressPool
+genBitString len =
+  (AddressPool . BA.fromList) `liftM` vectorOf len (elements [False, True])
 
 -- | Generate an arbitrary string consisting of '0' and '1' of the maximum given
 -- length.
-genBitStringMaxLen :: Int -> Gen String
+genBitStringMaxLen :: Int -> Gen AddressPool
 genBitStringMaxLen maxLen = choose (0, maxLen) >>= genBitString
 
 -- | Generator for config data with an empty cluster (no instances),
@@ -301,11 +327,15 @@ genEmptyCluster ncount = do
                     else GenericContainer nodemap
       continsts = GenericContainer Map.empty
       networks = GenericContainer Map.empty
+      disks = GenericContainer Map.empty
   let contgroups = GenericContainer $ Map.singleton guuid grp
   serial <- arbitrary
+  -- timestamp fields
+  ctime <- arbitrary
+  mtime <- arbitrary
   cluster <- resize 8 arbitrary
   let c = ConfigData version cluster contnodes contgroups continsts networks
-            serial
+            disks ctime mtime serial
   return c
 
 -- | FIXME: make an even simpler base version of creating a cluster.
@@ -343,6 +373,21 @@ prop_fillDict defaults custom =
                (fillDict d_map c_map (d_keys++c_keys) == Map.empty)
              ]
 
+prop_LogicalVolume_serialisation :: LogicalVolume -> Property
+prop_LogicalVolume_serialisation = testSerialisation
+
+prop_LogicalVolume_deserialisationFail :: Property
+prop_LogicalVolume_deserialisationFail =
+  conjoin . map (testDeserialisationFail (LogicalVolume "" "")) $
+    [ J.JSArray []
+    , J.JSString $ J.toJSString "/abc"
+    , J.JSString $ J.toJSString "abc/"
+    , J.JSString $ J.toJSString "../."
+    , J.JSString $ J.toJSString "g/snapshot"
+    , J.JSString $ J.toJSString "g/a_mimagex"
+    , J.JSString $ J.toJSString "g/r;3"
+    ]
+
 -- | Test that the serialisation of 'DiskLogicalId', which is
 -- implemented manually, is idempotent. Since we don't have a
 -- standalone JSON instance for DiskLogicalId (it's a data type that
@@ -352,6 +397,9 @@ prop_fillDict defaults custom =
 prop_Disk_serialisation :: Disk -> Property
 prop_Disk_serialisation = testSerialisation
 
+prop_Disk_array_serialisation :: Disk -> Property
+prop_Disk_array_serialisation = testArraySerialisation
+
 -- | Check that node serialisation is idempotent.
 prop_Node_serialisation :: Node -> Property
 prop_Node_serialisation = testSerialisation
@@ -360,6 +408,10 @@ prop_Node_serialisation = testSerialisation
 prop_Inst_serialisation :: Instance -> Property
 prop_Inst_serialisation = testSerialisation
 
+-- | Check that address pool serialisation is idempotent.
+prop_AddressPool_serialisation :: AddressPool -> Property
+prop_AddressPool_serialisation = testSerialisation
+
 -- | Check that network serialisation is idempotent.
 prop_Network_serialisation :: Network -> Property
 prop_Network_serialisation = testSerialisation
@@ -409,16 +461,13 @@ casePyCompatNetworks = do
   HUnit.assertEqual "Mismatch in number of returned networks"
     (length decoded) (length networks_with_properties)
   mapM_ (uncurry (HUnit.assertEqual "Different result after encoding/decoding")
-        ) $ zip decoded networks_with_properties
+        ) $ zip networks_with_properties decoded
 
 -- | Creates a tuple of the given network combined with some of its properties
 -- to be compared against the same properties generated by the python code.
 getNetworkProperties :: Network -> (Int, Int, Network)
 getNetworkProperties net =
-  let maybePool = createAddressPool net
-  in  case maybePool of
-           (Just pool) -> (getFreeCount pool, getReservedCount pool, net)
-           Nothing -> (-1, -1, net)
+    (getFreeCount net, getReservedCount net, net)
 
 -- | Tests the compatibility between Haskell-serialized node groups and their
 -- python-decoded and encoded version.
@@ -452,7 +501,7 @@ casePyCompatNodegroups = do
   HUnit.assertEqual "Mismatch in number of returned node groups"
     (length decoded) (length groups)
   mapM_ (uncurry (HUnit.assertEqual "Different result after encoding/decoding")
-        ) $ zip decoded groups
+        ) $ zip groups decoded
 
 -- | Generates a node group with up to 3 networks.
 -- | FIXME: This generates still somewhat completely random data, without normal
@@ -483,21 +532,28 @@ genNodeGroup = do
 instance Arbitrary NodeGroup where
   arbitrary = genNodeGroup
 
-$(genArbitrary ''Ip4Address)
+instance Arbitrary Ip4Address where
+  arbitrary = liftM mkIp4Address $ (,,,) <$> choose (0, 255)
+                                         <*> choose (0, 255)
+                                         <*> choose (0, 255)
+                                         <*> choose (0, 255)
 
 $(genArbitrary ''Ip4Network)
 
--- | Helper to compute absolute value of an IPv4 address.
-ip4AddrValue :: Ip4Address -> Integer
-ip4AddrValue (Ip4Address a b c d) =
-  fromIntegral a * (2^(24::Integer)) +
-  fromIntegral b * (2^(16::Integer)) +
-  fromIntegral c * (2^(8::Integer)) + fromIntegral d
+-- | Tests conversions of ip addresses from/to numbers.
+prop_ip4AddressAsNum :: Ip4Address -> Property
+prop_ip4AddressAsNum ip4 =
+  ip4AddressFromNumber (ip4AddressToNumber ip4) ==? ip4
 
--- | Tests that any difference between IPv4 consecutive addresses is 1.
-prop_nextIp4Address :: Ip4Address -> Property
-prop_nextIp4Address ip4 =
-  ip4AddrValue (nextIp4Address ip4) ==? ip4AddrValue ip4 + 1
+-- | Tests that the number produced by 'ip4AddressToNumber' has the correct
+-- order of bytes.
+prop_ip4AddressToNumber :: Word32 -> Property
+prop_ip4AddressToNumber w =
+  let byte :: Int -> Word32
+      byte i = (w `div` (256^i)) `mod` 256
+      ipaddr = List.intercalate "." $ map (show . byte) [3,2..0]
+  in ip4AddressToNumber <$> readIp4Address ipaddr
+     ==? (return (toInteger w) :: Either String Integer)
 
 -- | IsString instance for 'Ip4Address', to help write the tests.
 instance IsString Ip4Address where
@@ -546,61 +602,76 @@ casePyCompatInstances = do
   HUnit.assertEqual "Mismatch in number of returned instances"
     (length decoded) (length instances)
   mapM_ (uncurry (HUnit.assertEqual "Different result after encoding/decoding")
-        ) $ zip decoded instances
+        ) $ zip instances decoded
+
+-- | A helper function for creating 'LIDPlain' values.
+mkLIDPlain :: String -> String -> DiskLogicalId
+mkLIDPlain = (LIDPlain .) . LogicalVolume
 
 -- | Tests that the logical ID is correctly found in a plain disk
 caseIncludeLogicalIdPlain :: HUnit.Assertion
 caseIncludeLogicalIdPlain =
   let vg_name = "xenvg" :: String
       lv_name = "1234sdf-qwef-2134-asff-asd2-23145d.data" :: String
+      lv = LogicalVolume vg_name lv_name
+      time = TOD 0 0
       d =
-        Disk (LIDPlain vg_name lv_name) [] "diskname" 1000 DiskRdWr
-          Nothing Nothing "asdfgr-1234-5123-daf3-sdfw-134f43"
+        Disk (LIDPlain lv) [] "diskname" 1000 DiskRdWr
+          Nothing Nothing Nothing "asdfgr-1234-5123-daf3-sdfw-134f43"
+          0 time time
   in
     HUnit.assertBool "Unable to detect that plain Disk includes logical ID" $
-      includesLogicalId vg_name lv_name d
+      includesLogicalId lv d
 
 -- | Tests that the logical ID is correctly found in a DRBD disk
 caseIncludeLogicalIdDrbd :: HUnit.Assertion
 caseIncludeLogicalIdDrbd =
   let vg_name = "xenvg" :: String
       lv_name = "1234sdf-qwef-2134-asff-asd2-23145d.data" :: String
-      d = 
+      time = TOD 0 0
+      d =
         Disk
           (LIDDrbd8 "node1.example.com" "node2.example.com" 2000 1 5 "secret")
-          [ Disk (LIDPlain "onevg" "onelv") [] "disk1" 1000 DiskRdWr Nothing
-              Nothing "145145-asdf-sdf2-2134-asfd-534g2x"
-          , Disk (LIDPlain vg_name lv_name) [] "disk2" 1000 DiskRdWr Nothing
-              Nothing "6gd3sd-423f-ag2j-563b-dg34-gj3fse"
-          ] "diskname" 1000 DiskRdWr Nothing Nothing
-          "asdfgr-1234-5123-daf3-sdfw-134f43"
+          [ Disk (mkLIDPlain "onevg" "onelv") [] "disk1" 1000 DiskRdWr Nothing
+              Nothing Nothing "145145-asdf-sdf2-2134-asfd-534g2x" 0 time time
+          , Disk (mkLIDPlain vg_name lv_name) [] "disk2" 1000 DiskRdWr Nothing
+              Nothing Nothing "6gd3sd-423f-ag2j-563b-dg34-gj3fse" 0 time time
+          ] "diskname" 1000 DiskRdWr Nothing Nothing Nothing
+          "asdfgr-1234-5123-daf3-sdfw-134f43" 0 time time
   in
     HUnit.assertBool "Unable to detect that plain Disk includes logical ID" $
-      includesLogicalId vg_name lv_name d
+      includesLogicalId (LogicalVolume vg_name lv_name) d
 
 -- | Tests that the logical ID is correctly NOT found in a plain disk
 caseNotIncludeLogicalIdPlain :: HUnit.Assertion
 caseNotIncludeLogicalIdPlain =
   let vg_name = "xenvg" :: String
       lv_name = "1234sdf-qwef-2134-asff-asd2-23145d.data" :: String
+      time = TOD 0 0
       d =
-        Disk (LIDPlain "othervg" "otherlv") [] "diskname" 1000 DiskRdWr
-          Nothing Nothing "asdfgr-1234-5123-daf3-sdfw-134f43"
+        Disk (mkLIDPlain "othervg" "otherlv") [] "diskname" 1000 DiskRdWr
+          Nothing Nothing Nothing "asdfgr-1234-5123-daf3-sdfw-134f43"
+          0 time time
   in
     HUnit.assertBool "Unable to detect that plain Disk includes logical ID" $
-      not (includesLogicalId vg_name lv_name d)
+      not (includesLogicalId (LogicalVolume vg_name lv_name) d)
 
 testSuite "Objects"
   [ 'prop_fillDict
+  , 'prop_LogicalVolume_serialisation
+  , 'prop_LogicalVolume_deserialisationFail
   , 'prop_Disk_serialisation
+  , 'prop_Disk_array_serialisation
   , 'prop_Inst_serialisation
+  , 'prop_AddressPool_serialisation
   , 'prop_Network_serialisation
   , 'prop_Node_serialisation
   , 'prop_Config_serialisation
   , 'casePyCompatNetworks
   , 'casePyCompatNodegroups
   , 'casePyCompatInstances
-  , 'prop_nextIp4Address
+  , 'prop_ip4AddressAsNum
+  , 'prop_ip4AddressToNumber
   , 'caseNextIp4Address
   , 'caseIncludeLogicalIdPlain
   , 'caseIncludeLogicalIdDrbd
