@@ -43,6 +43,8 @@ import Test.HUnit
 import Data.Char (isSpace)
 import qualified Data.Either as Either
 import Data.List
+import Data.Maybe (listToMaybe)
+import qualified Data.Set as S
 import System.Time
 import qualified Text.JSON as J
 #ifndef NO_REGEX_PCRE
@@ -76,6 +78,21 @@ prop_commaSplitJoin :: String -> Property
 prop_commaSplitJoin s =
   commaJoin (sepSplit ',' s) ==? s
 
+-- | Test 'findFirst' on several possible inputs.
+prop_findFirst :: Property
+prop_findFirst =
+  forAll (genSublist [0..5 :: Int]) $ \xs ->
+  forAll (choose (-2, 7)) $ \base ->
+  printTestCase "findFirst utility function" $
+  let r = findFirst base (S.fromList xs)
+      (ss, es) = partition (< r) $ dropWhile (< base) xs
+      -- the prefix must be a range of numbers
+      -- and the suffix must not start with 'r'
+   in conjoin [ and $ zipWith ((==) . (+ 1)) ss (drop 1 ss)
+              , maybe True (> r) (listToMaybe es)
+              ]
+
+
 -- | fromObjWithDefault, we test using the Maybe monad and an integer
 -- value.
 prop_fromObjWithDefault :: Integer -> String -> Bool
@@ -333,27 +350,10 @@ prop_splitRecombineEithers es =
         (splitleft, splitright, trail) = splitEithers es
         emptylist = []::[Int]
 
--- | Test the update function for standard deviations against the naive
--- implementation.
-prop_stddev_update :: Property
-prop_stddev_update =
-  forAll (choose (0, 6) >>= flip vectorOf (choose (0, 1))) $ \xs ->
-  forAll (choose (0, 1)) $ \a ->
-  forAll (choose (0, 1)) $ \b ->
-  forAll (choose (1, 6) >>= flip vectorOf (choose (0, 1))) $ \ys ->
-  let original = xs ++ [a] ++ ys
-      modified = xs ++ [b] ++ ys
-      with_update = getStatisticValue
-                    $ updateStatistics (getStdDevStatistics original) (a,b)
-      direct = stdDev modified
-  in printTestCase ("Value computed by update " ++ show with_update
-                    ++ " differs too much from correct value " ++ show direct)
-                   (abs (with_update - direct) < 1e-12)
-
--- | Test list for the Utils module.
 testSuite "Utils"
             [ 'prop_commaJoinSplit
             , 'prop_commaSplitJoin
+            , 'prop_findFirst
             , 'prop_fromObjWithDefault
             , 'prop_if'if
             , 'prop_select
@@ -376,5 +376,4 @@ testSuite "Utils"
             , 'prop_chompPrefix_empty_string
             , 'prop_chompPrefix_nothing
             , 'prop_splitRecombineEithers
-            , 'prop_stddev_update
             ]
