@@ -65,9 +65,13 @@ import Test.Ganeti.JSON
 import Test.Ganeti.Jobs
 import Test.Ganeti.JQueue
 import Test.Ganeti.Kvmd
+import Test.Ganeti.Locking.Allocation
+import Test.Ganeti.Locking.Locks
+import Test.Ganeti.Locking.Waiting
 import Test.Ganeti.Luxi
 import Test.Ganeti.Network
 import Test.Ganeti.Objects
+import Test.Ganeti.Objects.BitArray
 import Test.Ganeti.OpCodes
 import Test.Ganeti.Query.Aliases
 import Test.Ganeti.Query.Filter
@@ -83,8 +87,12 @@ import Test.Ganeti.Storage.Drbd.Parser
 import Test.Ganeti.Storage.Drbd.Types
 import Test.Ganeti.Storage.Lvm.LVParser
 import Test.Ganeti.THH
+import Test.Ganeti.THH.Types
 import Test.Ganeti.Types
 import Test.Ganeti.Utils
+import Test.Ganeti.Utils.MultiMap
+import Test.Ganeti.Utils.Statistics
+import Test.Ganeti.WConfd.TempRes
 
 -- | Our default test options, overring the built-in test-framework
 -- ones (but not the supplied command line parameters).
@@ -129,9 +137,13 @@ allTests =
   , testJobs
   , testJQueue
   , testKvmd
+  , testLocking_Allocation
+  , testLocking_Locks
+  , testLocking_Waiting
   , testLuxi
   , testNetwork
   , testObjects
+  , testObjects_BitArray
   , testOpCodes
   , testQuery_Aliases
   , testQuery_Filter
@@ -144,8 +156,12 @@ allTests =
   , testSsconf
   , testStorage_Lvm_LVParser
   , testTHH
+  , testTHH_Types
   , testTypes
   , testUtils
+  , testUtils_MultiMap
+  , testUtils_Statistics
+  , testWConfd_TempRes
   ]
 
 -- | Main function. Note we don't use defaultMain since we want to
