@@ -36,6 +36,7 @@ import unittest
 import shutil
 import tempfile
 import operator
+import json
 
 from ganeti import constants
 from ganeti import utils
@@ -56,9 +57,14 @@ def GetMinimalConfig():
       "default_iallocator_params": {},
       "ndparams": {},
       "candidate_certs": {},
+      "install_image": "",
+      "instance_communication_network": "",
+      "zeroing_image": "",
+      "compression_tools": constants.IEC_DEFAULT_TOOLS,
       "enabled_user_shutdown": False,
     },
     "instances": {},
+    "disks": {},
     "networks": {},
     "nodegroups": {},
     "nodes": {
@@ -387,6 +393,9 @@ class TestCfgupgrade(unittest.TestCase):
   def testUpgradeFullConfigFrom_2_9(self):
     self._TestUpgradeFromFile("cluster_config_2.9.json", False)
 
+  def testUpgradeFullConfigFrom_2_10(self):
+    self._TestUpgradeFromFile("cluster_config_2.10.json", False)
+
   def testUpgradeCurrent(self):
     self._TestSimpleUpgrade(constants.CONFIG_VERSION, False)
 
@@ -404,12 +413,11 @@ class TestCfgupgrade(unittest.TestCase):
   def testDowngradeFullConfig(self):
     """Test for upgrade + downgrade combination."""
     # This test can work only with the previous version of a configuration!
-    oldconfname = "cluster_config_2.10.json"
+    oldconfname = "cluster_config_2.11.json"
     self._TestUpgradeFromFile(oldconfname, False)
     _RunUpgrade(self.tmpdir, False, True, downgrade=True)
     oldconf = self._LoadTestDataConfig(oldconfname)
     newconf = self._LoadConfig()
-
     self.assertEqual(oldconf, newconf)
 
   def testDowngradeFullConfigBackwardFrom_2_7(self):
