@@ -597,6 +597,12 @@ class _FakeConfigForComputeIPolicyInstanceViolation:
       constants.ND_EXCLUSIVE_STORAGE: self.excl_stor,
       }
 
+  def GetInstanceNodes(self, instance_uuid):
+    return ("pnode_uuid", )
+
+  def GetInstanceDisks(self, _):
+    return [objects.Disk(size=512, spindles=13, uuid="disk_uuid")]
+
 
 class TestComputeIPolicyInstanceViolation(unittest.TestCase):
   def test(self):
@@ -605,16 +611,17 @@ class TestComputeIPolicyInstanceViolation(unittest.TestCase):
       constants.BE_VCPUS: 2,
       constants.BE_SPINDLE_USE: 4,
       }
-    disks = [objects.Disk(size=512, spindles=13)]
     cfg = _FakeConfigForComputeIPolicyInstanceViolation(beparams, False)
-    instance = objects.Instance(beparams=beparams, disks=disks, nics=[],
+    instance = objects.Instance(beparams=beparams, disks=["disk_uuid"],
+                                nics=[], primary_node="pnode_uuid",
                                 disk_template=constants.DT_PLAIN)
     stub = _StubComputeIPolicySpecViolation(2048, 2, 1, 0, [512], 4,
                                             constants.DT_PLAIN)
     ret = common.ComputeIPolicyInstanceViolation(NotImplemented, instance,
                                                  cfg, _compute_fn=stub)
     self.assertEqual(ret, [])
-    instance2 = objects.Instance(beparams={}, disks=disks, nics=[],
+    instance2 = objects.Instance(beparams={}, disks=["disk_uuid"],
+                                 nics=[], primary_node="pnode_uuid",
                                  disk_template=constants.DT_PLAIN)
     ret = common.ComputeIPolicyInstanceViolation(NotImplemented, instance2,
                                                  cfg, _compute_fn=stub)
